
/*
*
* iscsi initiator driver
*
* Author: Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
*         Liran Schour, IBM Haifa Research Laboratory <lirans@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* iscsi.c
*
* SCSI driver main entry points.
*
* %I% %W% %D% %T%
*/

#ifndef __KERNEL__
#  define __KERNEL__
#endif /* __KERNEL__ */
#ifndef MODULE
#  define  MODULE
#endif /* MODULE */


#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/proc_fs.h>
#include <linux/blkdev.h>
#include <asm/uaccess.h>
#define __KERNEL_SYSCALLS__
#include <linux/unistd.h>   /* kernel_thread */
#include <linux/fs.h>
#include <linux/socket.h>
#include <linux/net.h>
#include <linux/mm.h>
#include <linux/sched.h>
#include <linux/blk.h>
#include <linux/interrupt.h>
#include <asm/page.h>
#include <asm/processor.h>
#include <asm/softirq.h>
#include <scsi.h>
#include <hosts.h>
#include <sd.h>
#include <scsi_obsolete.h>
#include "iscsi_interfaces.h"
#include "iscsi_local_trace.h"
#include "iscsi_core_types.h"
#include "iscsi_core_messages.h"
#include "iscsi_client.h"
#include "iscsi_device.h"
#include "iscsi_ioctl.h"
/*----------------------------------------------*/

/* global variables used by driver */

s32 sessionIndeces[ISCSI_MAX_TARGETS];
struct Scsi_Host *iSCSI_Host;
u32		configDone = 0;
Q_t		queuecommandQ;
iscsiLock_t	queuecommandLock;
iscsiEvent_t	queuecommandEvent;
iscsiEvent_t    cleanUpEvent;
u32		iscsiDriverExit = 0;
u32             lastReset=0;
#if ( LINUX_VERSION_CODE > KERNEL_VERSION(2,3,0) )
DECLARE_MUTEX_LOCKED(errorWait);
#else
struct semaphore errorWait = MUTEX_LOCKED;
#endif

/*----------------------------------------------*/

int iscsi_detect(Scsi_Host_Template *sht);
int iscsi_release(struct Scsi_Host *sh);
int iscsi_queuecommand(Scsi_Cmnd *sc, void (*done)(Scsi_Cmnd *));
int iscsi_queuecommand_helper(void *arg);
int iscsi_abort(Scsi_Cmnd *sc);
int iscsi_reset(Scsi_Cmnd *sc, unsigned int flags);
int iscsi_device_reset(Scsi_Cmnd *sc);
int iscsi_bus_reset(Scsi_Cmnd *sc);
int iscsi_target_reset(Scsi_Cmnd *sc);
int iscsi_bios_param(Disk *disk, kdev_t dev, int *ip);
int iscsi_error_handler(void * no_use);


/* imported function from core need to remove from here */
sessionInfo_t * getSessionStruct(u32 sessionHandle);
void breakTcpConnectionsAndEmptyQs(sessionInfo_t *si);
pendingCmd_t *removeCmdFromQ(Q_t *cmdQ, SCB_t *scb);
pendingCmd_t *findCmdInTable(sessionInfo_t *si, SCB_t *scb);
pendingCmd_t *removeCmdFromTable(sessionInfo_t *si, pendingCmd_t *cmd);

#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,3,27) )
static struct proc_dir_entry iscsi_proc_entry = {
	PROC_SCSI_SCSI,			/* low_ino */
	5,				/* namelen */
	"iscsi",			/* name */
	S_IFDIR|S_IRUGO|S_IXUGO,	/* mode */
	2				/* nlink */
};
#define ISCSI_PROC_ENTRY &iscsi_proc_entry
#else 
#define ISCSI_PROC_ENTRY NULL
#endif

static Scsi_Host_Template driver_template = {
	next:NULL,
	module:NULL,
	proc_dir:ISCSI_PROC_ENTRY,//&iscsi_proc_entry,
	proc_info:NULL,	/* can add here a function for reading and writing /proc file system */
	name:"iscsi",
	detect:iscsi_detect,
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 2, 16))
	revoke:NULL,
#endif /* KERNEL_VERSION */
	release:iscsi_release,
	info:NULL,
	ioctl:NULL,
	command:NULL,
	queuecommand:iscsi_queuecommand,
	eh_strategy_handler:NULL,
	/* reset device, SCSI bus, entire SCSI controller */
	eh_abort_handler:NULL,//iscsi_abort,
	eh_device_reset_handler:NULL,//iscsi_device_reset,
	eh_bus_reset_handler:NULL,//iscsi_bus_reset,
	eh_host_reset_handler:NULL,
	abort:NULL,	/* depreciated */
	reset:NULL,	/* depreciated */
	slave_attach:NULL,
	bios_param:iscsi_bios_param,
	can_queue:ISCSI_CAN_QUEUE,	/* max # commands we can queue */
	this_id:-1, /* initiator does not have SCSI id on this SCSI 'bus' */
	sg_tablesize:ISCSI_MAX_SG,//SG_NONE,	/* we don't currently support scatter-gather */
	cmd_per_lun:ISCSI_COMMANDS_PER_LUN,
	// cmd_per_lun:2,
	present:1,	/* number of iscsi cards */
	unchecked_isa_dma:0,
	use_clustering:ENABLE_CLUSTERING,
	use_new_eh_code:1,
	emulated:0,	/* act as if we are a real adapter */
};

/* use existing template for init_module and cleanup_module */
#include "scsi_module.c"

/*----------------------------------------------*/

/*
* NAME: config_helper
*
* PURPOSE: perform iscsi configuration; connections of sessions, etc
*
* PARAMETERS: none
*
* RETURNS: none
*
* CALLED BY: iscsi_detect as a separate thread
*
* NOTES:
* Read iscsi client configuration file.
* Make connections to targets specified in configuration file.
* Do not allow SCSI commands until we exit from this function.
*/
int config_helper(void *arg)
{
	s32	rc;
	s32	i;
	iscsiIrql_t irql;

	TRACE_ENTER;

	/* initialize array of session handles */
	for (i = 0; i < ISCSI_MAX_TARGETS; i++) {
		sessionIndeces[i] = -1;
	}

	rc = iscsiCoreInitData();
	if (rc < 0) {
		/* xxx could not initiatlize; log error */
		/* xxx shouldn't allow any iscsi operations */
		return 0;
	}

	PRMSG("before readConfigFile");
	rc = readConfigFile();
	PRMSG("after readConfigFile");
	PRINT(rc);

	PRINT(configDone);
	iscsiOSlock(&queuecommandLock, &irql);
	configDone = 1;
	iscsiOSwakeup(&queuecommandEvent);
	iscsiOSunlock(&queuecommandLock, irql);
	PRINT(configDone);

	TRACE_EXIT;
	return 0;
}

/*----------------------------------------------*/

/* main SCSI driver entry points */


/*
* NAME: iscsi_detect
*
* PURPOSE: initialize the iscsi driver
*
* PARAMETERS:
*	sht - SCSI Host Template structure, provided by system
*
* RETURNS:
*	1 for success (actually, number of HBAs of type iscsi)
*	0 for error
*
* CALLED BY: system
*
* NOTES:
* Perform global initializations.
* Read iscsi client configuration file.
* Make connections to targets specified in configuration file.
* A process environment is required in order to make file system calls and
* socket calls from the kernel.
* On an SMP, it seems that we might not be in a process environment, and we are
* limited in what functions we may call.
* We therefore create a kernel thread to perform most of the initiatlization.
*/
int iscsi_detect(Scsi_Host_Template *sht) {
	struct Scsi_Host *sh = NULL;
	s32 rc;

	TRACE_ENTER;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
	sht->proc_name = "iscsi";
#endif

	/* 2nd parameter is amount of private data for this adapter */
	sh = scsi_register(sht, 0);
	if (NULL == sh) {
		return 0;
	}

	/*
	Each "channel" can have "max_id" targets (wide SCSI),
	each of which can have up to "max_luns" LUs.
	Linux limits max_luns to 1 or 8,
	depending on some compilation flag.
	We use a different channel for each iscsi session,
	and allow a certain number of devices per session.
	We declare a maximum number of iscsi targets, even if there are currently less
	*/
	sh->max_channel = ISCSI_MAX_TARGETS - 1;
	sh->max_id = ISCSI_MAX_DEVICES_PER_TARGET;
	sh->max_lun = 1;

	/* Try and register character device associated with iscsi host */
	rc = register_chrdev(ISCSI_DEVICE_MAJOR, ISCSI_DEVICE_NAME, 
			     &iscsi_device_fops);
	if (rc < 0){
		// printk("iscsi: Failed to register character device\n");
		scsi_unregister(sh);
		return 0;
	}

	iSCSI_Host = sh;

	/* Are there other fields in sh that we have to fill in? */

	iscsiOSlockInit(&queuecommandLock);
	iscsiOSeventInit(&queuecommandEvent);
	Qinit(&queuecommandQ);

	/* create a thread for iscsi_queuecommand_helper(), to be used
	when commands come in at interrupt level */
	kernel_thread(iscsi_queuecommand_helper, NULL, 0);

	/* do login for each attached target */
	/* we perform this in a separate thread */
	/* xxx properly clean up the config_helper thread */
	kernel_thread(config_helper, NULL, 0);

	kernel_thread(iscsi_error_handler, NULL, 0);
	
	TRACE_EXIT;
	return 1;
}


/*
* NAME: iscsi_release
*
* PURPOSE: free up all iscsi driver resources
*
* PARAMETERS:
*	sh - SCSI Host structure obtained in iscsi_detect().
*
* RETURNS:
*	0 for success
*
* CALLED BY: system
*
* NOTES:
*/
int iscsi_release(struct Scsi_Host *sh) {
	int i;

	TRACE_ENTER;
	PRHEX(sh);

	iscsiOSeventInit(&cleanUpEvent);
	iscsiDriverExit = 1;
	
	/* xxx do we have to delay to ensure that the thread exits? */
	for (i = 0; i < ISCSI_MAX_TARGETS; i++) {
		if (-1 != sessionIndeces[i]) {
			iscsiCoreDeleteSession(sessionIndeces[i]);
		}
	}
	iscsiCoreCleanupData();
	up(&errorWait);
	iscsiOSsleep(&cleanUpEvent);
	scsi_unregister(sh);
	unregister_chrdev(ISCSI_DEVICE_MAJOR, ISCSI_DEVICE_NAME);
	TRACE_EXIT;
	return 0;
}

Scsi_Cmnd *scInError = NULL;

int iscsi_error_handler(void * no_use)
{
  sessionInfo_t *si;
  
  TRACE_ENTER;
  
  while (1 == 1) {

    down(&errorWait);

    if( iscsiDriverExit) {
      break;
    }
    
    if( scInError == NULL) continue;
    
    // we dont want any new cmd's now
    
    si = getSessionStruct(sessionIndeces[scInError->channel]);
    if (NULL == si) {
      // printk("ISCSI: Hard Error cant get session! lost cmd\n");
      return 0;
    }
    
    // ugly but it will do it for now
    if((jiffies - lastReset) > 120 * HZ) {
      PRMSG("ISCSI:Reset sessions");
      iscsiCoreResetSessions();
      lastReset = jiffies;
    }
   
  }
  iscsiOSwakeup(&cleanUpEvent);
  
  TRACE_EXIT;
  return 0;
}


void iscsi_queuecommand_timeout(unsigned long ptr)
{
  int rc;
  Scsi_Cmnd     *sc = (Scsi_Cmnd *)ptr;

  TRACE_ENTER;

  rc = iscsi_queuecommand(sc, sc->scsi_done);
  if(rc != 0){

    if( sc->eh_timeout.function != NULL )
      {
	del_timer(&sc->eh_timeout);
      }
    
    
    sc->eh_timeout.data = (unsigned long) sc;
    sc->eh_timeout.expires = jiffies + ISCSI_CMD_TIMEOUT;
    sc->eh_timeout.function = (void (*)(unsigned long))iscsi_queuecommand_timeout;
    
    // activate our timer
    add_timer(&sc->eh_timeout);
  }
  
  TRACE_EXIT;
}



void iscsi_cmd_timeout(unsigned long ptr) 
{
  Scsi_Cmnd       *sc = (Scsi_Cmnd *)ptr;
  sessionInfo_t	  *si;
  u32             flags; 

  
  TRACE_ENTER;

  si = getSessionStruct(sessionIndeces[sc->channel]);
  if (NULL == si) {
    PRMSG("ISCSI:Hard Error Lost cmd\n");
    return;
  }
  
  spin_lock_irqsave(&io_request_lock, flags);
 
  if( sc->eh_timeout.function != NULL )
    {
      del_timer(&sc->eh_timeout);
    }
  
  sc->eh_timeout.data = (unsigned long) sc;
  sc->eh_timeout.expires = jiffies + ISCSI_CMD_TIMEOUT;
  sc->eh_timeout.function = (void (*)(unsigned long))iscsi_cmd_timeout;
  // activate our timer
  add_timer(&sc->eh_timeout);
  
  scInError = sc; 
  // wake up handler
  up(&errorWait);
 
  spin_unlock_irqrestore(&io_request_lock, flags);
  
  TRACE_EXIT;
  return;
}


/*
* NAME: iscsi_queuecommand2
*
* PURPOSE: Do all of the work for iscsi_queuecommand
*
* PARAMETERS:
*	sc - SCSI Command Block
*
* RETURNS:
*	0 - command was processed (either successfully or with an error)
*	-1 - catastrophic error
*
* CALLED BY: iscsi_queuecommand
*
* NOTES:
* We are guarenteed to be running at passive level (no interrupts).
*/
int iscsi_queuecommand2(Scsi_Cmnd *sc)
{
	u32		flags;
	s32		rc;
	iscsiLUN_t	lun;
	senseData_t	senseData;

	TRACE_ENTER;
	PRHEX(sc);
	PRINT(sc->channel);
	PRINT(sc->target);
	
	if (sc->channel >= ISCSI_MAX_TARGETS) {
		/* LOGICAL UNIT NOT SUPPORTED (0x25, 0x00) */
		PRMSG("illegal channel");
		SET_SENSE(senseData, ILLEGAL_REQUEST, 0x25, 0x00);
		iscsiOSputSense(sc, senseData, sizeof(senseData_t));
		sc->result = CHECK_CONDITION;
		sc->scsi_done(sc);
		TRACE_EXIT;
		return 0;

	}
	/* verify that session is active */
	if (-1 == sessionIndeces[sc->channel]) {
		/* LOGICAL UNIT NOT SUPPORTED (0x25, 0x00) */
		PRMSG("session not active");
		SET_SENSE(senseData, ILLEGAL_REQUEST, 0x25, 0x00);
		iscsiOSputSense(sc, senseData, sizeof(senseData_t));
		sc->result = CHECK_CONDITION;
		sc->scsi_done(sc);
		TRACE_EXIT;
		return 0;
	}
	memset(lun, 0, sizeof(iscsiLUN_t));
	lun[0] = LOGICAL_UNIT_ADDRESS_METHOD;
	lun[1] = sc->target;
	PRHEX(lun[1]);
	PRHEX(sc->cmnd[0]);

	/* determine if have DATA_IN or DATA_OUT */
	if (!sc->bufflen) {
		flags = 0;
	}
	else if (NULL == sc->buffer) {
		/* no place to put data */
		// printk("iscsi: XXX buffer is NULL, but buflen = %d \n", sc->bufflen);
		/* xxx fix this */
		return -1;
	}
	else {
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) )		  
	  /* for all commands that send data, set DATA_OUT */
	  switch(sc->sc_data_direction) {
	                case SCSI_DATA_WRITE:
				flags = ISCSI_DATA_OUT;
				break;
	                case SCSI_DATA_READ:
				flags = ISCSI_DATA_IN;
				break;
			default:
				flags = 0;
				break;
	}
#else
		switch(sc->cmnd[0]) {
			/* commands that neither send nor receive data to/from target */
			case ISCSI_SCSIOP_TEST_UNIT_READY:
			case ISCSI_SCSIOP_REZERO_UNIT:
			case ISCSI_SCSIOP_WRITE_FILEMARKS:
			case ISCSI_SCSIOP_SPACE:
			case ISCSI_SCSIOP_ERASE:
			case ISCSI_SCSIOP_START_STOP_UNIT:
			case ISCSI_SCSIOP_ALLOW_MEDIUM_REMOVAL:
				flags = 0;
				break;

			/* SCSI commands that send data to target */
			case ISCSI_SCSIOP_FORMAT_UNIT:
			case ISCSI_SCSIOP_REASSIGN_BLOCKS:
			case ISCSI_SCSIOP_WRITE_6:
			case ISCSI_SCSIOP_MODE_SELECT_6:
			case ISCSI_SCSIOP_RESERVE:
			case ISCSI_SCSIOP_COPY:
			case ISCSI_SCSIOP_SEND_DIAGNOSTIC:
			case ISCSI_SCSIOP_SET_WINDOW:
			case ISCSI_SCSIOP_WRITE_10:
			case ISCSI_SCSIOP_WRITE_AND_VERIFY:
			case ISCSI_SCSIOP_SEARCH_DATA_HIGH:
			case ISCSI_SCSIOP_SEARCH_DATA_EQUAL:
			case ISCSI_SCSIOP_SEARCH_DATA_LOW:
			case ISCSI_SCSIOP_COMPARE:
			case ISCSI_SCSIOP_MEDIUM_SCAN:
			case ISCSI_SCSIOP_COPY_AND_VERIFY:
			case ISCSI_SCSIOP_WRITE_BUFFER:
			case ISCSI_SCSIOP_UPDATE_BLOCK:
			case ISCSI_SCSIOP_WRITE_LONG:
			case ISCSI_SCSIOP_CHANGE_DEFINITION:
			case ISCSI_SCSIOP_WRITE_SAME:
			case ISCSI_SCSIOP_LOG_SELECT:
			case ISCSI_SCSIOP_XDWRITE:
			case ISCSI_SCSIOP_XPWRITE:
			case ISCSI_SCSIOP_MODE_SELECT_10:
			case ISCSI_SCSIOP_XDWRITE_EXTENDED:
			case ISCSI_SCSIOP_REBUILD:
			case ISCSI_SCSIOP_WRITE_AND_VERIFY_12:
			case ISCSI_SCSIOP_SEARCH_DATA_HIGH_12:
			case ISCSI_SCSIOP_SEARCH_DATA_EQUAL_12:
			case ISCSI_SCSIOP_SEARCH_DATA_LOW_12:
			case ISCSI_SCSIOP_SEND_VOLUME_TAG:
			case ISCSI_SCSIOP_WRITE_12:
			//case ISCSI_SCSIOP_EA:
				flags = ISCSI_DATA_OUT;
				break;
			default:
				flags = ISCSI_DATA_IN;
				break;
		}
#endif
	}

	rc = iscsiCoreEnterCmdInQ(sc, sc->cmnd, sc->cmd_len, sessionIndeces[sc->channel], lun, sc->request_buffer, sc->request_bufflen, sc->use_sg, flags);


	TRACE_EXIT;
	return rc;
}

/*
* NAME: iscsi_queuecommand_helper
*
* PURPOSE: intermediate routine for iscsi_queuecommand for special cases
*
* PARAMETERS:
*	arg - not used
*
* RETURNS: none
*
* CREATED BY: iscsi_detect
*
* USED BY: iscsi_queuecommand
*
* NOTES:
* Call iscsi_queuecommand2 to perform all of the actual work.
*/
int iscsi_queuecommand_helper(void *arg)
{
	iscsiIrql_t irql;
	cmd_entry_t *ce;
	s32 rc;

	TRACE_ENTER;

	PRINT(configDone);
	iscsiOSlock(&queuecommandLock, &irql);
	while ((!configDone) && (!iscsiDriverExit)) {
		/* unlock and sleep atomically */
		iscsiOSunlockAndSleepAndLock(&queuecommandLock, &queuecommandEvent, &irql);
	}
	iscsiOSunlock(&queuecommandLock, irql);
	PRINT(configDone);

	while (!iscsiDriverExit) {
		iscsiOSlock(&queuecommandLock, &irql);
		Qget(&queuecommandQ, ce, cmd_entry_t);
		if (NULL == ce) {
			break;
		}
		else {
			iscsiOSunlock(&queuecommandLock, irql);
			rc = iscsi_queuecommand2(ce->sc);
			if (rc != 0) {
				iscsiOScompleteCommand(ce->sc, ISCSI_SCSISTAT_BUSY);
			}
			kfree(ce);
		}
	}
	Qfree(&queuecommandQ);
	iscsiOSeventDestroy(queuecommandEvent);

	TRACE_EXIT;
	return 0;
}

/*
* NAME: iscsi_queuecommand
*
* PURPOSE: enter a SCSI command into a Q to be processed
*
* PARAMETERS:
*	sc - SCSI Command Block
*	done - pointer to completion routine
*
* RETURNS:
*	0 - command was processed (either successfully or with an error)
*	-1 - catastrophic error
*
* CALLED BY: system
*
* NOTES:
* Call iscsi_queuecommand2 to perform all of the actual work.
* If we are running at priority level, we simply create a new kernel thread
* perform iscsi_queuecommand2.
*/
int iscsi_queuecommand(Scsi_Cmnd *sc, void (*done)(Scsi_Cmnd *))
{
	s32	rc;
	cmd_entry_t *ce = NULL;
	iscsiIrql_t irql;

	TRACE_ENTER;
	sc->scsi_done = done;

	// first disable system timer
	if( sc->eh_timeout.function != NULL )
	  {
	    del_timer(&sc->eh_timeout);
	  }

	
	sc->eh_timeout.data = (unsigned long) sc;
	sc->eh_timeout.expires = jiffies + ISCSI_CMD_TIMEOUT;
	sc->eh_timeout.function = (void (*)(unsigned long))iscsi_cmd_timeout;

	//XXX DEBUG
	sc->SCp.Status= 0;
	
	// activate our timer
	add_timer(&sc->eh_timeout);

	/* we sometimes want to defer processing of the command */
	PRINT(configDone);
	if (!configDone) {
		ce = kmalloc(sizeof(cmd_entry_t), GFP_KERNEL);
		if (NULL == ce) {
			TRACE_EXIT;
			return -ENOMEM;
		}
		ce->sc = sc;
		Qput(&queuecommandQ, ce, cmd_entry_t);
		iscsiOSlock(&queuecommandLock, &irql);
		iscsiOSwakeup(&queuecommandEvent);
		iscsiOSunlock(&queuecommandLock, irql);
		TRACE_EXIT;
		return 0;
	}

	rc = iscsi_queuecommand2(sc);
#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) )	
	if( rc != 0) {
	  if( sc->eh_timeout.function != NULL )
	    {
	      del_timer(&sc->eh_timeout);
	    } 
	}
#endif

	TRACE_EXIT;
	return rc;
}



/*
* NAME: iscsi_abort
*
* PURPOSE: abort specified SCSI command
*
* PARAMETERS:
*	sc - SCSI Command Block to abort
*
* RETURNS:
*	SCSI_ABORT_SUCCESS - command was successfully aborted
*	SCSI_ABORT_PENDING - command is in process of being aborted
*	SCSI_ABORT_NOT_RUNNING - command not found
*	SCSI_ABORT_ERROR - illegal session or other error
*
* CALLED BY: system
*
* NOTES:
* According to what it says in scsi_error.c, we are allowed to block
* inside the abort operation until the operation is completed.
* Note: the io_request_lock is held when this function is called.
* This means that no other block device operations will occur as long
* this lock is held.
*
* FUTURE_ENHANCEMENTS:
* We also want to ensure that we don't sleep forever while waiting for
* a target to respond. We should therefore set a timer to cancel the
* Abort operation if it takes too long.
*/
int iscsi_abort(Scsi_Cmnd *sc) {
	u32	rc;
	u32	rc2;

	TRACE_ENTER;
	PRHEX(sc);

	if (sc->channel >= ISCSI_MAX_TARGETS) {
		return FAILED;
	}
	if (-1 == sessionIndeces[sc->channel]) {
		return FAILED;
	}
	// printk("iscsi: entering iscsi_abort\n");
	PRINT(sc->channel);
	PRINT(sc->target);
	PRHEX(sc->cmnd[0]);
	rc = iscsiCoreAbortCmd(sc, NULL, sessionIndeces[sc->channel], 0);
	PRINT(rc);

	/* Translate between iscsi return codes and linux return codes. */
	rc2 = 0;
	switch(rc) {
		case ISCSI_ABORT_SUCCESS:
			rc2 = SUCCESS;
			break;
		default:
			rc2 = FAILED;
			break;
	}
	TRACE_EXIT;
	// printk("iscsi: exiting iscsi_abort\n");
	return rc2;
}



/*
* NAME: iscsi_device_reset
*
* PURPOSE: reset a device
*
* PARAMETERS:
*	sc - SCSI Command Block (that caused problem with device)
*
* RETURNS:
*	SCSI_RESET_SUCCESS - successfully reset the device
*	SCSI_RESET_PENDING - reset operation in progress
*	SCSI_RESET_ERROR - illegal session or other error
*
* CALLED BY: system
*
* NOTES:
*/
int iscsi_device_reset(Scsi_Cmnd *sc) {
	u32 rc;
	u32 rc2;
	iscsiLUN_t lun;

	TRACE_ENTER;
	PRHEX(sc);
	if (sc->channel >= ISCSI_MAX_TARGETS) {
		TRACE_EXIT;
		return FAILED;
	}
	if (-1 == sessionIndeces[sc->channel]) {
		TRACE_EXIT;
		return FAILED;
	}
	PRINT(sc->channel);
	PRINT(sc->target);
	// printk("iscsi: entering iscsi_device_reset \n");
	memset(lun, 0, sizeof(iscsiLUN_t));
	lun[0] = LOGICAL_UNIT_ADDRESS_METHOD;
	lun[1] = sc->target;
	// rc = iscsiCoreResetDevice(sessionIndeces[sc->channel], sc, lun, ISCSI_WAIT_FOR_COMPLETION);
	rc = iscsiCoreResetDevice(sessionIndeces[sc->channel], sc, lun, 0);

	/* Translate between iscsi return codes and linux return codes. */
	rc2 = 0;
	switch(rc) {
		case ISCSI_RESET_SUCCESS:
			rc2 = SUCCESS;
			break;
		default:
			rc2 = FAILED;
			break;
	}
	// printk("iscsi: exiting iscsi_device_reset \n");
	PRINT(rc2);

	TRACE_EXIT;
	return rc2;
}


/*
* NAME: iscsi_bus_reset
*
* PURPOSE: reset entire session
*
* PARAMETERS:
*	sc - SCSI Command Block (that caused problem)
*
* RETURNS:
*	SCSI_RESET_SUCCESS - successfully reset the session
*	SCSI_RESET_PENDING - reset operation in progress
*	SCSI_RESET_ERROR - illegal session or other error
*
* CALLED BY: system
*
* NOTES:
*/
int iscsi_bus_reset(Scsi_Cmnd *sc) {
	u32 rc = 0;
	u32 rc2 = 0;

	TRACE_ENTER;
	PRHEX(sc);
	if (sc->channel >= ISCSI_MAX_TARGETS) {
		return FAILED;
	}
	if (-1 == sessionIndeces[sc->channel]) {
		return FAILED;
	}
	PRINT(sc->channel);
	PRINT(sc->target);
	// printk("iscsi: entering iscsi_bus_reset \n");
	 rc = iscsiCoreResetBus(sessionIndeces[sc->channel], NULL, ISCSI_WAIT_FOR_COMPLETION);
	
	/* Translate between iscsi return codes and linux return codes. */
	rc2 = 0;
	switch(rc) {
		case ISCSI_RESET_SUCCESS:
			rc2 = SUCCESS;
			break;
		default:
			rc2 = FAILED;
			break;
	}
	// printk("iscsi: exiting iscsi_bus_reset \n");
	TRACE_EXIT;
	return rc2;
}


/*
* NAME: iscsi_target_reset
*
* PURPOSE: reset entire target
*
* PARAMETERS:
*	sc - SCSI Command Block (that caused problem)
*
* RETURNS:
*	SCSI_RESET_SUCCESS - successfully reset the session
*	SCSI_RESET_PENDING - reset operation in progress
*	SCSI_RESET_ERROR - illegal session or other error
*
* CALLED BY: system
*
* NOTES:
*/
int iscsi_target_reset(Scsi_Cmnd *sc) {
	u32 rc;
	u32 rc2;

	TRACE_ENTER;
	PRHEX(sc);
	/* xxx */
	// iscsiCoreResetSessions();
	// return SUCCESS;
	if (sc->channel >= ISCSI_MAX_TARGETS) {
		return FAILED;
	}
	if (-1 == sessionIndeces[sc->channel]) {
		return FAILED;
	}
	PRINT(sc->channel);
	PRINT(sc->target);
	// printk("iscsi: entering iscsi_target_reset \n");
	// rc = iscsiCoreResetTarget(sessionIndeces[sc->channel], sc, ISCSI_WAIT_FOR_COMPLETION);
	rc = iscsiCoreResetTarget(sessionIndeces[sc->channel], sc, 0);

	/* Translate between iscsi return codes and linux return codes. */
	rc2 = 0;
	switch(rc) {
		case ISCSI_RESET_SUCCESS:
			rc2 = SUCCESS;
			break;
		default:
			rc2 = FAILED;
			break;
	}
	// printk("iscsi: exiting iscsi_target_reset \n");
	TRACE_EXIT;
	return rc2;
}



/*
* NAME: iscsi_bios_param
*
* PURPOSE: provide disk geometry
*
* PARAMETERS:
*	disk - some disk parameters obtained from queying the disk
*	dev - device number for specified disk
*	ip - array of disk parameters that we must fill in
*
* RETURNS:
*	0 for success
*
* CALLED BY: system
*
* NOTES:
* We provide dummy disk geometry values, similar to other SCSI drivers
* in Linux.
* These numbers provide the size of blocks that can be allocated to
* disk partitions using fdisk.
* The smallest partitionable size is (heads * sectors * block_size)
* where block_size is typically 512 bytes.
*
* FUTURE_ENHANCEMENTS:
* In a future version, it might make sense to obtain actual numbers
* for the specific disk using MODE_SENSE information.
*/
int iscsi_bios_param(Disk *disk, kdev_t dev, int *ip){
	/* just to make the OS happy */
	TRACE_ENTER;
	PRINT(disk->capacity);
	ip[0] = 64; /* heads */
	ip[1] = 32; /* sectors */
	ip[2] = disk->capacity / (ip[0] * ip[1]); /* cylinders */
	if (ip[2] > 1024) { /* big disk */
		ip[0] = 255;
		ip[1] = 63;
		ip[2] = disk->capacity / (ip[0] * ip[1]);
	}
	TRACE_EXIT;
	return 0;
}

/*----------------------------------------------*/

/*
* NAME: iscsiOScompleteCommand
*
* PURPOSE: call scsi_done() to complete command processing.
*
* PARAMETERS:
*	scb - SCSI Command Block of completed command
*	status - SCSI status of completed command
*
* RETURNS: none
*
* CALLED BY: many functions, but mainly by handleScsiResponse()
*
* NOTES:
* The system expects that scsi_done() is called from an interupt handler
* and that the io_request_lock is held.
*/
void iscsiOScompleteCommand(SCB_t *scb, u32 status)
{
	u32 flags;

#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0) )
	// delete our timer
	del_timer(&scb->eh_timeout);
	scb->eh_timeout.data = (unsigned long) NULL;
	scb->eh_timeout.function = NULL;
	
#endif
	/* we have to obtain the io_request_lock before calling done() */
	spin_lock_irqsave(&io_request_lock, flags);
	// that will improve performence when target is busy
	if(ISCSI_SCSISTAT_BUSY == status) {
	  status = QUEUE_FULL << 1;
	}
	scb->result = status;
	scb->scsi_done(scb);
	spin_unlock_irqrestore(&io_request_lock, flags);
}


/*
* NAME: iscsiOScompleteCommandAbort
*
* PURPOSE: complete command that was aborted.
*
* PARAMETERS:
*	scb - SCSI Command Block of aborted command
*
* RETURNS: none
*
* CALLED BY: handleTaskManResponse(), completeTaskManLocal(), iscsiCoreAbortCmd()
*
* NOTES:
*/
void iscsiOScompleteCommandAbort(SCB_t *scb)
{
   // printk("entering iscsiOScompleteCommandAbort\n");
	scb->result = (DID_ABORT << 16);
	// printk("exiting iscsiOScompleteCommandAbort\n");
}


/*
* NAME: iscsiOScompleteCommandReset
*
* PURPOSE: complete command due to a reset operation
*
* PARAMETERS:
*	scb - SCSI Command Block of completed command
*
* RETURNS: none
*
* CALLED BY: removeCmdsResetLun(), cleanupPendingOpsTables(), emptySessionQs(), iscsiCoreResetDevice()
*
* NOTES:
*/
void iscsiOScompleteCommandReset(SCB_t *scb)
{
  sessionInfo_t *si;
  
  TRACE_ENTER;
 
  si = getSessionStruct(sessionIndeces[scb->channel]);
  if (NULL != si) {
    si->cmdRN--;
  }
  
  if( scb->eh_timeout.function != NULL )
    {
      del_timer(&scb->eh_timeout);
    }
  
  
  scb->eh_timeout.data = (unsigned long) scb;
  scb->eh_timeout.expires = jiffies + ISCSI_CMD_TIMEOUT;
  scb->eh_timeout.function = (void (*)(unsigned long))iscsi_queuecommand_timeout;
  
  // activate our timer
  add_timer(&scb->eh_timeout);

  
  TRACE_EXIT;
}

/*
* NAME: iscsiOSunlockAndSleepAndLock
*
* PURPOSE: release a lock and go to sleep "atomically"; re-acquire lock when wake up
*
* PARAMETERS:
*	lock - the lock to release before goint to sleep
*	event - the event on which to go to sleep
*
* RETURNS: none
*
* CALLED BY: 
*
* NOTES:
* This routine is called with the lock held.
* We return from this call with the lock held.
* The routine that wakes up on the specified event should obtain
* the lock before awakening, so that the event won't get lost.
*/
void iscsiOSunlockAndSleepAndLock(iscsiLock_t *lock, iscsiEvent_t *event, iscsiIrql_t *irql)
{
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0) )
        DECLARE_WAITQUEUE(wait, current);
#else
        struct wait_queue wait = { current, NULL };
#endif
	current->state = TASK_UNINTERRUPTIBLE;
	add_wait_queue(event, &wait);
	iscsiOSunlock(lock, *irql);
	schedule();
	iscsiOSlock(lock, irql);
	remove_wait_queue(event, &wait);
}

