
/*
*
* iscsi initiator driver
*
* Author: Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*                               
* iscsi_client.h
*
* Some definitions and prototypes for iscsi initiator driver.
*
* %I% %W% %D% %T%
*/

#ifndef ISCSI_CLIENT_H
#define ISCSI_CLIENT_H

#include "iscsi_local_types.h"


/* data structure for saving commands before TCP connections are established */
typedef struct cmd_entry {
	struct cmd_entry *forw;
	Scsi_Cmnd *sc;
} cmd_entry_t;

extern s32 readConfigFile(void);
extern s32 sessionIndeces[];

extern struct Scsi_Host *iSCSI_Host;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,7)
#define ISCSI_MAX_SG 8
#else
#define ISCSI_MAX_SG 32
#endif

#define ISCSI_CMD_TIMEOUT 8 * HZ
#define ISCSI_READ_TIMEOUT 1 * HZ
#define ISCSI_RESET_INTERVAL 120 * HZ

#define LOGICAL_UNIT_ADDRESS_METHOD	0x80

#define ISCSI_CONFIG_FILE "/etc/iscsiclient.conf"


/* SCSI commands that neither send nor receive data to/from target */
#define ISCSI_SCSIOP_TEST_UNIT_READY	0x00
#define ISCSI_SCSIOP_REZERO_UNIT	0x01
#define ISCSI_SCSIOP_WRITE_FILEMARKS	0x10
#define ISCSI_SCSIOP_SPACE		0x11
#define ISCSI_SCSIOP_ERASE		0x19
#define ISCSI_SCSIOP_START_STOP_UNIT	0x1b
#define ISCSI_SCSIOP_ALLOW_MEDIUM_REMOVAL	0x1e

/* SCSI commands that send data to target */
#define ISCSI_SCSIOP_FORMAT_UNIT	0x04
#define ISCSI_SCSIOP_REASSIGN_BLOCKS	0x07
#define ISCSI_SCSIOP_WRITE_6		0x0a
#define ISCSI_SCSIOP_MODE_SELECT_6	0x15
#define ISCSI_SCSIOP_RESERVE		0x16
#define ISCSI_SCSIOP_COPY		0x18
#define ISCSI_SCSIOP_SEND_DIAGNOSTIC	0x1d
#define ISCSI_SCSIOP_SET_WINDOW		0x24
#define ISCSI_SCSIOP_WRITE_10		0x2a
#define ISCSI_SCSIOP_WRITE_AND_VERIFY	0x2e
#define ISCSI_SCSIOP_VERIFY		0x2f
#define ISCSI_SCSIOP_SEARCH_DATA_HIGH	0x30
#define ISCSI_SCSIOP_SEARCH_DATA_EQUAL	0x31
#define ISCSI_SCSIOP_SEARCH_DATA_LOW	0x32
#define ISCSI_SCSIOP_COMPARE		0x39
#define ISCSI_SCSIOP_MEDIUM_SCAN	0x38
#define ISCSI_SCSIOP_COPY_AND_VERIFY	0x3a
#define ISCSI_SCSIOP_WRITE_BUFFER	0x3b
#define ISCSI_SCSIOP_UPDATE_BLOCK	0x3d
#define ISCSI_SCSIOP_WRITE_LONG		0x3f
#define ISCSI_SCSIOP_CHANGE_DEFINITION	0x40
#define ISCSI_SCSIOP_WRITE_SAME		0x41
#define ISCSI_SCSIOP_LOG_SELECT		0x4c
#define ISCSI_SCSIOP_XDWRITE		0x50
#define ISCSI_SCSIOP_XPWRITE		0x51
#define ISCSI_SCSIOP_MODE_SELECT_10	0x55
#define ISCSI_SCSIOP_XDWRITE_EXTENDED	0x80
#define ISCSI_SCSIOP_REBUILD		0x81
#define ISCSI_SCSIOP_WRITE_AND_VERIFY_12	0xae
#define ISCSI_SCSIOP_SEARCH_DATA_HIGH_12	0xb0
#define ISCSI_SCSIOP_SEARCH_DATA_EQUAL_12	0xb1
#define ISCSI_SCSIOP_SEARCH_DATA_LOW_12	0xb2
#define ISCSI_SCSIOP_SEND_VOLUME_TAG	0xb6
#define ISCSI_SCSIOP_WRITE_12		0xaa
//#define ISCSI_SCSIOP_EA		0xea


#endif /* ISCSI_CLIENT_H */
