
/*
*
* iscsi initiator driver
*
* Author: Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* iscsi_core_messages.h
*
* All kinds of constants used in iscsi message encoding/decoding.
*
* %I% %W% %D% %T%
*/

#ifndef ISCSI_CORE_MESSAGES_H
#define ISCSI_CORE_MESSAGES_H

#define ISCSI_HEADER_LEN	48

#define ISCSI_MESSAGE_MASK	0x7f
#define ISCSI_OPCODE(header) ((header->buf)[0] & ISCSI_MESSAGE_MASK)

/* iscsi opcodes */
#define ISCSI_NOP_MESSAGE	0x00
#define ISCSI_SCSI_COMMAND	0x01
#define ISCSI_TASK_MANAGEMENT	0x02
#define ISCSI_LOGIN_COMMAND	0x03
#define ISCSI_TEXT_COMMAND	0x04
#define ISCSI_DATA_WRITE	0x05
#define ISCSI_PING_MESSAGE	0x09
#define ISCSI_MAP_MESSAGE	0x0a
#define ISCSI_NOP_RESPONSE	0x40
#define ISCSI_SCSI_RESPONSE	0x41
#define ISCSI_TASK_RESPONSE	0x42
#define ISCSI_LOGIN_RESPONSE	0x43
#define ISCSI_TEXT_RESPONSE	0x44
#define ISCSI_DATA_READ		0x45
#define ISCSI_RTT		0x46
#define ISCSI_ASYNC_EVENT	0x47
#define ISCSI_BAD_OPCODE	0x48
#define ISCSI_PING_RESPONSE	0x49
#define ISCSI_MAP_RESPONSE	0x4a

#define ISCSI_RESPONSE_BIT	0x40
#define ISCSI_RETRY_BIT		0x80
#define ISCSI_POLL_BIT		0x80
#define ISCSI_READ_BIT		0x20

#define ISCSI_UNDERFLOW_BIT	0x01
#define ISCSI_OVERFLOW_BIT	0x02
#define ISCSI_STATUS_BIT	0x04

/* login types */
#define NO_AUTHENTICATION	0
#define IMPLICIT_LOGIN		1
#define CLEAR_TEXT		2
#define RSA_1_WAY		3
#define RSA_2_WAY		4


/* login status values */
#define ACCEPT_LOGIN	0
#define REJECT_LOGIN	1
#define ADDITIONAL_AUTHENTICATION_REQUIERED	2
#define REJECT_RECOVERY	3


/* task management response values */
#define FUNCTION_COMPLETE	0
#define FUNCTION_REJECTED	1


/* byte locations in iscsi header */
#define OP_CODE_BYTE		0
#define FLAGS_BYTE		1
#define FUNCTION_BYTE		1
#define LENGTH_BYTE		4
#define LUN_FIRST_BYTE		8
#define DATA_BUFFER_OFFSET_BYTE	8
#define DATA_TRANSFER_TAG	12
#define TASK_TAG_BYTE		16
#define ISID_BYTE		16
#define TSID_BYTE		18
#define REFERENCED_TAG_BYTE	20
#define RESIDUAL_COUNT_BYTE	20
#define EXPECTED_DATA_XFER_LEN	20
#define DESIRED_DATA_XFER_LEN	20
#define STAT_RN_BYTE		24
#define RTT_BUFFER_OFFSET_BYTE	24
#define CMD_RN_BYTE		24
#define EXP_CMD_RN_BYTE		28
#define EXP_STAT_RN_BYTE	28
#define MAX_CMD_RN_BYTE		32
#define CDB_FIRST_BYTE		32
#define STATUS_BYTE		36
#define TARGET_TRANSFER_TAG	36
#define RESPONSE_LEN_BYTE	40
#define SENSE_LEN_BYTE		42


/* bit values for flags in various iscsi headers */
#define R_BIT			0x20



/*
 *  Various scsi sense codes, status codes, and operation codes.
 *  These are defined here using new code names, since the predefined
 *  code names are incompatible in the different OS platforms. 
 */

#define ISCSI_SCSISENSE_NOT_READY	0x02
#define ISCSI_SCSISENSE_UNIT_ATTENTION	0x06
#define ISCSI_SCSISENSE_ABORTED_COMMAND	0x0b

#define ISCSI_SCSISTAT_GOOD		0x00
#define ISCSI_SCSISTAT_CHECK_CONDITION	0x02
#define ISCSI_SCSISTAT_CONDITION_MET	0x04
#define ISCSI_SCSISTAT_BUSY		0x08
#define ISCSI_SCSISTAT_INTERMEDIATE	0x10
#define ISCSI_SCSISTAT_INTERMEDIATE_MET	0x14
#define ISCSI_SCSISTAT_RESERVATION_CONFLICT	0x18
#define ISCSI_SCSISTAT_TASK_SET_FULL	0x28
#define ISCSI_SCSISTAT_ACA_ACTIVE	0x30

#endif /* ISCSI_CORE_MESSAGES_H */
