
/*
*
* iscsi initiator driver
*
* Author: Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* iscsi_core_query.h
*
* Data structures used to query the iscsi initiator driver.
*
* %I% %W% %D% %T%
*/

#ifndef ISCSI_CORE_QUERY_H
#define ISCSI_CORE_QUERY_H


typedef enum connState {
	CONNECTION_NOT_CONNECTED = 0,
	CONNECTION_INITIALIZING,
	CONNECTION_CONNECTED,
	CONNECTION_LOGIN_REJECTED,
	CONNECTION_LOGIN_ERROR,
	CONNECTION_CONNECT_FAILED,
	CONNECTION_ERROR,
	CONNECTION_BAD
} connState_t;


typedef struct iscsiQueryIo {
	u32	sessionNumber;
	void	*buf;
} iscsiQueryIo_t;


typedef struct queryConnection {
	u32	connectionNumber;
	u32	addr;
	u16	portNum;
	connState_t	state;
} queryConnection_t;


typedef struct querySession {
	u32	cmdRN;
	u32	expCmdRN;
	u32	maxCmdRN;
	u32	numMaxConnections;
	u32	numActiveConnections;
	u16	isid;
	u16	tsid;
	u32	nCmdQentries;
	u32	nRttQentries;
	u32	nTmQentries;
	u32	nPendCmds;
	u32	nPendTms;
} querySession_t;

typedef struct sessionStats {
	u32	nCmdsQueued;
	u32	nCmdsSent;
	u32	nCmdsReceived;
	u32	nCmdsAborted;
	u32	nCmdsReset;
	u32	nCmdsNotFound;
	u32	nCmdsBusy;
	u32	nCmdsNoConnection;
	u32	nDataSent;
	u32	nDataReceived;
	u32	nRttsQueued;
	u32	nRttsReceived;
	u32	nRttsRejected;
	u32	nCheckConditionReceived;
	u32	nBusyReceived;
	u32	nOtherStatusReceived;
	u32	nTMsReceived;
	u32	nAbortsQueued;
	u32	nAbortsSent;
	u32	nAbortsReceived;
	u32	nResetsSent;
	u32	nResetsReceived;
	u32	nResetDevice;
	u32	nResetBus;
	u32	nResetTarget;
	u32	nAbortCmd;
	u32	nResponsesDiscarded;
	u32	nBadTaskId;
	u32	nRejectOpCodeReceived;
	u32	nBadOpCodeReceived;
} sessionStats_t;

#ifdef __KERNEL__

extern s32 iscsiCoreQuerySession(u32 sessionHandle, querySession_t *qs);
extern s32 iscsiCoreQueryConnection(u32 sessionHandle, queryConnection_t *qs);
extern s32 iscsiCoreGetSessionStats(u32 sessionHandle, sessionStats_t *stats);

#endif /* __KERNEL__ */

#ifdef GATHER_STATISTICS

#define ISCSI_INCREMENT_COUNTER(si, counter) \
{ \
	iscsiIrql_t tirql; \
	iscsiOSlock(&si->statsLock, &tirql); \
	si->stats->counter++; \
	iscsiOSunlock(&si->statsLock, tirql); \
}

#else /* GATHER_STATISTICS */

#define ISCSI_INCREMENT_COUNTER(si, counter)

#endif /* GATHER_STATISTICS */


#endif /* ISCSI_CORE_QUERY_H */
