
/*
*
* iscsi initiator driver
*
* Author: Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* iscsi_core_queue.h
*
* Generic queue operations used by iscsi driver.
*
* %I% %W% %D% %T%
*/

#ifndef ISCSI_CORE_QUEUE_H
#define ISCSI_CORE_QUEUE_H

#include "iscsi_local_types.h"


typedef struct queue {
	iscsiLock_t	lock;
	void		*head;
	void		*tail;
	u32		count;
} Q_t;

/* initialize a Q */
#define Qinit(Q) \
{ \
	iscsiOSlockInit(&((Q)->lock)); \
	(Q)->head = (Q)->tail = NULL; \
	(Q)->count = 0; \
}


/* assumes Q is already empty */
#define Qfree(Q) \
{ \
	iscsiOSlockDestroy(&((Q)->lock)); \
}

/* insert an item at end of Q */
#define Qput(Q, item, type) \
{ \
	iscsiIrql_t tirql; \
	iscsiOSlock(&((Q)->lock), &tirql); \
	if (NULL == (Q)->head) { \
		(Q)->head = item; \
	} \
	else { \
		((type *) (Q)->tail)->forw = item; \
	} \
	(Q)->tail = item; \
	item->forw = NULL; \
	(Q)->count++; \
	iscsiOSunlock(&((Q)->lock), tirql); \
}

/* remove an item from head of Q */
#define Qget(Q, item, type) \
{ \
	if (NULL == (Q)->head) { \
		item = NULL; \
	} \
	else { \
		iscsiIrql_t tirql; \
		iscsiOSlock(&((Q)->lock), &tirql); \
		if (NULL == (Q)->head) { \
			item = NULL; \
		} \
		else { \
			item = (Q)->head; \
			(Q)->head = item->forw; \
			item->forw = NULL; \
			if (NULL == (Q)->head) { \
				(Q)->tail = NULL; \
			} \
			(Q)->count--; \
		} \
		iscsiOSunlock(&((Q)->lock), tirql); \
	} \
}


#endif /* ISCSI_CORE_QUEUE_H */
