
/*
*
* iscsi initiator driver
*
* Author: Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE..
*
* iscsi_core_types.h
*
* Data structures used by iscsi driver - OS independent.
*
* %I% %W% %D% %T%
*/

#ifndef ISCSI_CORE_TYPES_H
#define ISCSI_CORE_TYPES_H


#include "iscsi_local_types.h"
#include "iscsi_core_queue.h"
#include "iscsi_core_query.h"
#include "iscsi_interfaces.h"


#define ISCSI_HEADER_SIZE		48

typedef struct iscsiHeader {
	struct	iscsiHeader *forw;	/* used for links in queues */
	u8	buf[ISCSI_HEADER_SIZE];
} iscsiHeader_t;


typedef struct connInfo {	/* one such structure per TCP connection */
	iscsiSock_t		*sock;
	struct sessionInfo	*sInfo;
	s32			readThreadId;
	u32			addr;
	u16			portNum;
	connState_t		state;
} connInfo_t;


typedef struct pendingCmd {	/* one such structure for each command that was sent to target */
	struct pendingCmd	*forw;
	struct pendingCmd	*back;
	iscsiLUN_t		lun;
	SCB_t			*scb;
	uchar			*cdb;
	u32			cdbLen;
	struct sessionInfo	*sInfo;
	iscsiHeader_t		*header;
	uchar			*data;
	u32			datalen;
   u32         use_sg;
   connInfo_t  *connInfo;
	u32			taskId;
	u32			flags; /* DATA_IN, DATA_OUT */
} pendingCmd_t;

#define	CMD_WAITING_ON_EVENT	0x01

typedef struct pendCmdTable {
	pendingCmd_t		base;
	pendingCmd_t		*head;
	iscsiLock_t		lock;
} pendCmdTable_t;

typedef struct rttInfo {
	struct rttInfo		*forw;
	pendingCmd_t		*pendingCmd;
	iscsiHeader_t		*rttHeader;
} rttInfo_t;

typedef struct taskManageInfo {
	struct taskManageInfo	*forw;
	struct taskManageInfo	*back;
	SCB_t			*oldScb;
	SCB_t			*newScb;
	iscsiLUN_t		lun;
	pendingCmd_t		*pendingCmd;
	iscsiHeader_t		*header;
	connInfo_t		*connInfo;
	iscsiEvent_t		event;
	u32			flags;
	u32			taskId;
	u32			result;
	u8			operation;
	u8			pad[3];
} taskManageInfo_t;

/* pending Task Management table */
typedef struct pendTaskManTable {
	taskManageInfo_t	base;
	taskManageInfo_t	*head;
	iscsiLock_t		lock;
} pendTaskManTable_t;

typedef struct sessionInfo {	/* one such structure per session (target) */
	iscsiLock_t		lock;
	u32			cmdRN;
	u32			expCmdRN;
	u32			maxCmdRN;
	u32			expStatRN;
	connInfo_t		*connections;
	connInfo_t		**activeConnections;
	u32			numMaxConnections;
	u32			numActiveConnections;
	Q_t			cmdQ;
	Q_t			taskManQ;
	Q_t			rttQ;
	iscsiEvent_t		cmdQEvent;
	iscsiEvent_t		exitEvent;
	s32			pendingCmdThreadId;
	pendCmdTable_t		pendCmdTable;
	pendTaskManTable_t	pendTaskManTable;
	uchar			*loginParams;
	u32			loginParamsLen;
	u16			flags;
	u16			isid;
	u16			tsid;
	iscsiLock_t		statsLock;
	sessionStats_t		*stats;
} sessionInfo_t;

/* flags for sessionInfo */
#define SESSION_MUST_EXIT	0x1
#define SESSION_RESETTING	0x2

/* types of objects for which se set task id */
#define ISCSI_TASKID_COMMAND		0x01
#define ISCSI_TASKID_TASK_MANGEMENT	0x02
#define ISCSI_TASKID_REMOVE		0x10000

/* task management operations */
#define ABORT_TASK		1
#define ABORT_TASK_SET		2
#define CLEAR_ACA		3
#define CLEAR_TASK_SET		4
#define LOGICAL_UNIT_RESET	5
#define TARGET_RESET		6

/* sense data stuff */
#define SENSE_DATA_LEN		18	
#define SENSE_KEY_BYTE		2
#define ADD_SENSE_CODE_BYTE	12
#define ADD_SENSE_CODE_QUAL_BYTE	13

typedef char senseData_t[SENSE_DATA_LEN];

#define SET_SENSE(senseBuf, key, asc, ascq) \
{ \
	memset(senseBuf, 0, sizeof(senseData_t)); \
	senseBuf[0] = 0x70; \
	senseBuf[SENSE_KEY_BYTE] = key; \
	senseBuf[ADD_SENSE_CODE_BYTE] = asc; \
	senseBuf[ADD_SENSE_CODE_QUAL_BYTE] = ascq; \
}


#define SAME_LUN(lun1, lun2) (!memcmp(lun1, lun2, sizeof(iscsiLUN_t)))

#define SAME_STRING(str1, str2, len) (!memcmp(str1, str2, len))


/* common core global data structures */
extern Q_t freeCmdQ;
extern Q_t freeTaskManQ;
extern Q_t freeRttQ;
extern Q_t freeMsgHdrQ;


#endif /* ISCSI_CORE_TYPES_H */
