
/*
*
* iscsi initiator driver
*
* Author: Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* iscsi_interfaces.h
*
* Interfaces and data structures of iscsi common core,
* used by device driver interface routines.
*
* %I% %W% %D% %T%
*/

#ifndef ISCSI_INTERFACES_H
#define ISCSI_INTERFACES_H


#include "iscsi_local_types.h"


#define ISCSI_LUN_SIZE			8
#define ISCSI_MAX_TARGETS		16
#define ISCSI_MAX_DEVICES_PER_TARGET	16
#define ISCSI_CAN_QUEUE			64
#define ISCSI_MAX_TASK_MANAGEMENT	80

#ifdef Windows
#define ISCSI_COMMANDS_PER_LUN		16
#else
#define ISCSI_COMMANDS_PER_LUN		8
#endif


typedef unsigned char	uchar;
typedef u8	iscsiLUN_t[ISCSI_LUN_SIZE];


/* flags for iscsiCoreEnterCmdInQ() */
#define	ISCSI_DATA_IN	0x01
#define	ISCSI_DATA_OUT	0x02


/* flags for Task Management Operations */
#define	ISCSI_WAIT_FOR_COMPLETION	0x01
#define ISCSI_KEEP_CONNECTION_ALIVE 0x02


/* return codes for task management operations */
#define ISCSI_ABORT_SUCCESS		1
#define ISCSI_ABORT_PENDING		2
#define ISCSI_ABORT_CMD_NOT_FOUND	4
#define ISCSI_ABORT_ERROR		5


/* xxx do we have to differentiate between RESET_BUS, RESET_DEVICE, etc? */
#define ISCSI_RESET_SUCCESS		2
#define ISCSI_RESET_PENDING		3
#define ISCSI_RESET_ERROR		6


/* common core prototypes */
extern s32	iscsiCoreInitData(void);
extern void	iscsiCoreCleanupData(void);
extern s32	iscsiCoreCreateSession(u32 addr, u16 portNum, u32 nConnections, uchar *loginParams, u32 loginParamsLen);
extern s32	iscsiCoreDeleteSession(u32 sessionHandle);
extern void	iscsiCoreResetSessions(void);
extern s32	iscsiCoreEnterCmdInQ(SCB_t *scb, void *cdb, u32 cdbLen,  u32 sessionhandle, iscsiLUN_t lun, void *data, u32 datalen, u32 use_sg, u32 flags);
extern u32	iscsiCoreAbortCmd(SCB_t *oldScb, SCB_t *newScb, u32 sessionHandle, u32 flags);
extern u32	iscsiCoreResetDevice(u32 sessionHandle, SCB_t *scb, iscsiLUN_t lun, u32 flags);
extern u32	iscsiCoreResetBus(u32 sessionHandle, SCB_t *scb, u32 flags);
extern u32	iscsiCoreResetTarget(u32 sessionHandle, SCB_t *scb, u32 flags);
extern s32	iscsiCoreChangeSession(u32 index, uchar *loginParams, u32 loginParamsLen);


#endif /* ISCSI_INTERFACES_H */
