
/*
*
* iscsi initiator driver
*
* Author: Uri Schonfeld, IBM Haifa Research Laboratory <shuri@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* iscsi_ioctl.h
*
* iscsi initiator ioctl-s.
*
* %I% %W% %D% %T%
*/

#ifndef ISCSI_IOCTL_H

#define ISCSI_IOCTL_H

#include <linux/fs.h> /* struct file */
#include <linux/ioctl.h>


#ifdef __KERNEL__

#include "iscsi_local_types.h"

#else //__KERNEL__

typedef signed char s8;
typedef unsigned char u8;
 
typedef signed short s16;
typedef unsigned short u16;
 
typedef signed int s32;
typedef unsigned int u32;
 
typedef signed long long s64;
typedef unsigned long long u64;

#endif //__KERNEL__


#include "iscsi_core_query.h"

/* A struct describing all the details needed to add a new target */
typedef struct target_entry{
  u32    addr;
  u16    portNum;
  u32    nConnections;
  unsigned char *loginParams;
  u32    loginParamsLen;
} Target_Entry;


/* Use 'i' as magic number (iSCSI)*/
#define ISCSI_IOC_MAGIC  'i'

/*********************************************************************
 * Adding a device:
 * * add an appropriate define
 * * increase the ISCSI_IOC_MAXNR definition
 *********************************************************************/
#define ISCSI_IOCTL_ADD_TARGET	_IOW(ISCSI_IOC_MAGIC, 0, Target_Entry)
#define ISCSI_IOCTL_DEL_TARGET	_IOW(ISCSI_IOC_MAGIC, 1, int)
#define ISCSI_IOCTL_RESET_BUS	_IOW(ISCSI_IOC_MAGIC , 2, int)
#define ISCSI_IOCTL_RESET_DRIVER	_IO(ISCSI_IOC_MAGIC , 3)
#define ISCSI_IOCTL_QUERY_SESSION 	_IOWR(ISCSI_IOC_MAGIC , 4, iscsiQueryIo_t*)
#define ISCSI_IOCTL_SESSION_COUNTERS 	_IOWR(ISCSI_IOC_MAGIC , 5, iscsiQueryIo_t*)
#define ISCSI_IOCTL_QUERY_CONNECTION 	_IOWR(ISCSI_IOC_MAGIC , 6, iscsiQueryIo_t*)


/* The number equal to the max command number */
#define ISCSI_IOC_MAXNR 2

#ifdef __KERNEL__

extern int iscsi_ioctl(struct inode *, struct file *, unsigned int, 
		       unsigned long);
#endif //__KERNEL__

#endif //ISCSI_IOCTL_H
