
/*
*
* iscsi initiator driver
*
* Author: Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* iscsi_local_trace.h
*
* Trace macros, OS specific.
*
* %I% %W% %D% %T%
*/

#ifndef ISCSI_LOCAL_TRACE_H
#define ISCSI_LOCAL_TRACE_H

#ifdef TRACE

extern int tracing_on;

#ifdef __KERNEL__

#include <asm/current.h>
#include <linux/sched.h>

extern void kprintf(char *str);
extern void kprintf1(char *format, int arg1);
extern void kprintf2(char *format, int arg1, int arg2);
extern void kprintf3(char *format, int arg1, int arg2, int arg3);
extern void prekprintf(char * format, int timeStamp, int proc, char *file, int line, char *func);
extern void ksync(void);

#define TRACE_PRE_FORMAT "%u %u %s %d %s "
#define PRDO(prn) { \
	prekprintf(TRACE_PRE_FORMAT, jiffies, current->pid, __FILE__, __LINE__, __FUNCTION__); \
	prn; }

#define TRCAST int

#else /* __KERNEL__ not defined */

#include <stdio.h>
#include <unistd.h>

#define kprintf		printf
#define kprintf1	printf
#define kprintf2	printf
#define kprintf3	printf
#define prekprintf	printf

#define TRACE_PRE_FORMAT "%u %ld %s %d %s "
#define PRDO(prn) { \
	prekprintf(TRACE_PRE_FORMAT, getpid(), pthread_self(), __FILE__, __LINE__, __FUNCTION__); \
	(prn); \
	fflush(stdout); \
	}

#define TRCAST char *

#endif /* __KERNEL__ */


#define PR(prn) { if (tracing_on) PRDO(prn); }
#define PR0(format) PR(kprintf(format))
#define PR1(format, arg1) PR(kprintf1(format, arg1))
#define PR2(format, arg1, arg2) PR(kprintf2(format, arg1, arg2))
#define PR3(format, arg1, arg2, arg3) PR(kprintf3(format, arg1, arg2, arg3))

/* print various types of parameters */
#define PRINT(num) PR1(#num" = %d\n", num)
/* xxx #define PRINTL(num) PR1(#num" = %ld\n", num) */
#define PRHEX(hex) PR1(#hex" = 0x%x\n", ((int)hex))
#define PRSTR(str) PR1(#str" = \"%s\"\n", (TRCAST) str)
#define PRMSG(msg) PR0(#msg"\n")
#define PRCHAR(char) PR1(#char" = %c\n", char)

/* print contents of a buffer */
#define PRBUF(ptr, size) \
{ \
	int i; \
	if (tracing_on) { \
		PR1(#ptr"[0..%d] = ...\n", size); \
		for (i = 0; i < (size); i++) \
			PR3("... [%d] ... %d  0x%x\n", i, ((uchar*) ptr)[i], ((uchar*) ptr)[i]); \
	} \
}

#define TRACE_ENTER PR0("entering function \n");
#define TRACE_EXIT PR0("exiting function \n");

#else /* TRACE */

#define PRINT(num)
#define PRHEX(num)
#define PRSTR(str)
#define PRMSG(msg)
#define PRBUF(ptr, size)
#define PRINTL(num)
#define PRCHAR(char)
#define ksync()

#define TRACE_ENTER
#define TRACE_EXIT

#define PR(prn)
#define PR0(format)
#define PR1(format, arg1)
#define PR2(format, arg1, arg2)
#define PR3(format, arg1, arg2, arg3)

#endif /* TRACE */


#endif /* ISCSI_LOCAL_TRACE_H */
