
/*
*
* iscsi initiator driver
*
* Author: Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* iscsi_local_types.h
*
* Linux specific data types needed for iscsi driver.
*
* %I% %W% %D% %T%
*/

#ifndef ISCSI_LOCAL_TYPE_H
#define ISCSI_LOCAL_TYPE_H


#include <asm/types.h>
#include <linux/sched.h>
#ifndef LINUX_VERSION_CODE
#include <linux/version.h>
#endif
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0) )
#include <asm/semaphore.h>
#else
#include <asm/spinlock.h>
#endif
#include <asm/processor.h>
#include <linux/wait.h>
#include <net/sock.h>
#include <linux/socket.h>
#include <linux/malloc.h>
#include <linux/kernel.h>
#include <linux/blkdev.h>
#include <scsi.h>

typedef spinlock_t			iscsiLock_t;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
typedef wait_queue_head_t               iscsiEvent_t;
#else
typedef struct wait_queue*		iscsiEvent_t;
#endif
typedef u32				iscsiIrql_t;
typedef struct scsi_cmnd		SCB_t;

/* information we save for a socket structure */
typedef struct iscsiSock {
	struct sockaddr_in	sin;
	struct semaphore	sem;
	struct socket		*sock;
        iscsiEvent_t            event;
        u32			status;
        u32                     threadIsActive;
        struct task_struct      *threadId;
} iscsiSock_t;

#define SOCK_STATUS_INVALID	1
#define SOCK_STATUS_UNCONNECTED	2
#define SOCK_STATUS_CONNECTED   4
#define SOCK_STATUS_CONNECTING  8

typedef int iscsiThreadEntry;
#define threadReturn	0

#define iscsiOSmalloc(size)		kmalloc(size, GFP_KERNEL)
#define iscsiOSfree(ptr)		kfree(ptr)
#define iscsiOSlockInit(lock)		spin_lock_init(lock)
#define iscsiOSlock(lock, irql)		spin_lock_irqsave(lock, (*irql))
#define iscsiOSunlock(lock, irql)	spin_unlock_irqrestore(lock, irql)
#define iscsiOSlockDestroy(lock)
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
#define iscsiOSeventInit(event)         init_waitqueue_head(event)
#else	
#define iscsiOSeventInit(event)		((*event) = NULL)
#endif
#define iscsiOSeventDestroy(event)
#define iscsiOSsleep(event)		sleep_on(event)
#define iscsiOSwakeup(event)		wake_up(event)
#define iscsiOScreateThread(fcn, arg)	kernel_thread(fcn, arg, 0)
#ifndef MIN
#define MIN(a, b)	(((a) < (b)) ? (a) : (b))
#endif /* MIN */
#define iscsiOSputSense(scb, ptr, len)	memcpy(scb->sense_buffer, ptr, MIN(len, sizeof(scb->sense_buffer)))

#define iscsiOScompleteTaskMan(operation, newscb, result)



/* OS dependent prototypes */
extern s32 iscsiOSreadFromConnection(iscsiSock_t *sock, void *buffer, u32 len, u32 use_sg, u32 offset);
extern s32 iscsiOSreadFromConnection2(iscsiSock_t *sock, void *buffer, u32 len, u32 use_sg, u32 offset);
extern s32 iscsiOSwriteToConnection(iscsiSock_t *sock, void *buffer, u32 len, u32 use_sg, u32 offset);
extern s32 iscsiOSallocateConnection(u32 addr, u16 portNum, iscsiSock_t **isock);
extern s32 iscsiOSmakeConnection(u32 addr, u16 portNum, iscsiSock_t *isock);
extern void iscsiOSbreakConnection(iscsiSock_t *isock);
extern void iscsiOSdestroyConnection(iscsiSock_t *isock);
extern void iscsiOScompleteCommand(SCB_t *scb, u32 status);
extern void iscsiOScompleteCommandAbort(SCB_t *scb);
extern void iscsiOScompleteCommandReset(SCB_t *scb);
extern void iscsiOSunlockAndSleepAndLock(iscsiLock_t *lock, iscsiEvent_t *event, iscsiIrql_t *irql);



#endif /* ISCSI_LOCAL_TYPE_H */
