
/*
*
* iscsi initiator driver
*
* Authors: 
* Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
* John Marberg, IBM Haifa Research Laboratory <marberg@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* iscsi_local_types.h
*
* OS-specific datatypes and functions used by the initiator common core.
*
* Definitions in this file must be tailored for the given OS,
* and the file renamed to iscsi_local_types.h
*/

#ifndef ISCSI_LOCAL_TYPES_H
#define ISCSI_LOCAL_TYPES_H

/*
 *  Insert here #include lines for any header files
 *  that may be required for the compilation of the common core.
 *
 */

/* #include ???? */

/*
 *  Platform independent scalar types used by the common core.
 *  Uncomment and complete any type definition that is not
 *  predefined in some system header file that can be included above.
 */

/* typedef ????  u8;  */   /*  8 bit unsigned */
/* typedef ????  s8;  */   /*  8 bit signed   */
/* typedef ????  u16; */   /* 16 bit unsigned */
/* typedef ????  s16; */   /* 16 bit signed   */
/* typedef ????  u32; */   /* 32 bit unsigned */
/* typedef ????  s32; */   /* 32 bit signed   */


/*
 *  OS-specific iSCSI data types used by the common core.
 *  Complete the definitions according to the requirements of the OS.
 */

typedef ???? iscsiLock_t;
typedef ???? iscsiEvent_t;
typedef ???? SCB_t;
typedef ???? iscsiSock_t;
typedef ???? iscsiIrql_t;
typedef ???? iscsiThreadEntry;


/*
 * OS-specific macros used by the common core.
 * Complete the macros according to the requirements of the OS.
 */


/*
 * Return value from the body of a kernel thread.
 * This is a value of type iscsiThreadEntry, 
 * which is OS-specific and defined in this file.
 * If iscsiThreadEntry is defined as void, 
 * the macro threadReturn should be defined as empty.
 */

#define threadReturn


/*
 *  OS-specific iSCSI function prototypes used by the common core.
 *  Any fuction ptototype may be replaced by a macro.
 */

extern void	*iscsiOSmalloc(u32 size);
extern void  iscsiOSfree(void *ptr);

extern void  iscsiOSlockInit(iscsiLock_t *lock);
extern void  iscsiOSlock(iscsiLock_t *lock, iscsiIrql_t *irql);
extern void  iscsiOSunlock(iscsiLock_t *lock, iscsiIrql_t irql);
extern void  iscsiOSunlockAndSleep(iscsiLock_t *lock, iscsiEvent_t *event, iscsiIrql_t *irql);
extern void  iscsiOSlockDestroy(iscsiLock_t *lock);

extern void  iscsiOSeventInit(iscsiEvent_t *event);
extern void  iscsiOSeventDestroy(iscsiEvent_t *event);
extern void  iscsiOSsleep(iscsiEvent_t *event);
extern void  iscsiOSwakeup(iscsiEvent_t *event);

extern s32   iscsiOScreateThread(iscsiThreadEntry (*fname)(void *), void *arg);

extern void  iscsiOSputSense(SCB_t *scb, void *senseData, u32 senseLen);
extern void  iscsiOScompleteCommand(SCB_t *scb, u32 status);
extern void  iscsiOScompleteCommandAbort(SCB_t *scb);
extern void  iscsiOScompleteCommandReset(SCB_t *scb);
extern void  iscsiOScompleteTaskMan(u8 operation, SCB_t *scb, u32 result);

extern s32   iscsiOSreadFromConnection(iscsiSock_t *sock, void *buffer, u32 len);
extern s32   iscsiOSreadFromConnection2(iscsiSock_t *sock, void *buffer, u32 len);
extern s32   iscsiOSwriteToConnection(iscsiSock_t *sock, void *buffer, u32 len);
extern s32   iscsiOSallocateConnection(u32 addr, u16 portNum, iscsiSock_t **sock);
extern s32   iscsiOSmakeConnection(u32 addr, u16 portNum, iscsiSock_t *sock);
extern void  iscsiOSbreakConnection(iscsiSock_t *sock);
extern void  iscsiOSdestroyConnection(iscsiSock_t *sock);



#endif /* ISCSI_LOCAL_TYPES_H */
