
/*
*
* iscsi initiator driver
*
* Author: Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* misc.c
*
* Miscelaneous routines.
*	iscsiCoreInitData()
*	iscsiCoreCleanupData()
*
* %I% %W% %D% %T%
*/
#include "iscsi_core_types.h"
#include "iscsi_core_prot.h"




/*
* NAME: iscsiCoreInitData
*
* PURPOSE: initialize iscsi driver data structures
*
* PARAMETERS: none
*
* RETURNS:
*	0 - success
*	-1 - failure
*
* CALLED BY: OS dependent driver routine
*
* NOTES:
*/
s32 iscsiCoreInitData(void) {
	initGlobalQs();
	initSessions();
	return 0;
}


/*
* NAME: iscsiCoreCleanupData
*
* PURPOSE: clean up iscsi driver data structures
*
* PARAMETERS: none
*
* RETURNS: none
*
* CALLED BY: OS dependent driver routine
*
* NOTES:
*/
void iscsiCoreCleanupData(void) {
	cleanupSessions();
	cleanupGlobalQs();
	return;
}



/*
* NAME: GT_WITH_WRAP
*
* PURPOSE: check whether first parameter is greater than second, taking 32-bit wrap into consideration
*
* PARAMETERS:
*	a, b - variables to check against one another
*
* RETURNS:
*	1 - if first number is greater than second number, taking wrap around into consideration.
*	0 - otherwise
*
* CALLED BY: iscsiCoreEnterCmdInQ(), handleScsiResponse()
*
* NOTES:
* a and b are assumed to be of type u32.
* Furthurmore, it is assumed that a and b will never be
* further apart than WRAP_LEEWAY.
*/
#ifndef UINT_MAX
#define UINT_MAX 0xffffffff
#endif /* UINT_MAX */
#define WRAP_LEEWAY 0x10000000
u32 GT_WITH_WRAP(u32 a, u32 b)
{
	if (((a) > (b)) && (((a) < (UINT_MAX - WRAP_LEEWAY)) || ((b) > WRAP_LEEWAY))) {
		return 1;
	}
	else if (((a) < WRAP_LEEWAY) && ((b) > (UINT_MAX - WRAP_LEEWAY))) {
		return 1;
	}
	else {
		return 0;
	}
}

