
/*
*
* iscsi initiator driver
*
* Author: Kalman Meth, IBM Haifa Research Laboratory <meth@il.ibm.com>
*
* Copyright (c) IBM Corporation 2001 
* Licensed Material - Program Property of IBM
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 3. Neither the name of IBM Corporation nor the names of its contributors
*    may be used to endorse or promote products derived from this software
*    without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
* ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
* A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
* HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* trace.c
*
* Routines to save traces of kernel execution.
*
* %I% %W% %D% %T%
*/

#ifndef __KERNEL__
#define __KERNEL__
#endif /* __KERNEL__ */

#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/file.h>
#include <asm/uaccess.h>
#include "iscsi_local_types.h"
#include "iscsi_local_trace.h"

int tracing_on = 1;

#ifdef TRACE

void kprintf(char *str) {
	char filename[128];
	struct file *fptr;
	int len;
	int rc;
	mm_segment_t oldfs;

#ifdef __SMP__
	printk("%s", str);
	return;
#endif /* __SMP__ */
	sprintf(filename, "/tmp/iscsi/trace/trace.%d", current->pid);
	fptr = filp_open(filename, O_CREAT | O_WRONLY | O_APPEND, 0666);
	if (IS_ERR(fptr)) {
		return;
	}

	len = strlen(str);
	oldfs = get_fs();
	set_fs(get_ds()); /* buffer is in kernel space */
	rc = fptr->f_op->write(fptr, str, len, &fptr->f_pos);
	set_fs(oldfs);

	filp_close(fptr, current->files);
}

void ksync(void)
{
	struct file *fptr;
	char filename[128];

	fptr = filp_open(filename, O_CREAT | O_WRONLY | O_APPEND, 0666);
	if (IS_ERR(fptr)) {
		return;
	}

#if ( LINUX_VERSION_CODE < KERNEL_VERSION(2,3,0) )
	fptr->f_op->fsync(fptr, fptr->f_dentry);
#else
	fptr->f_op->fsync(fptr, fptr->f_dentry, 0);
#endif
	filp_close(fptr, current->files);
}

void prekprintf(char * format, int timeStamp, int proc, char *file, int line, char *func) {
	char str[512];

	sprintf(str, format, timeStamp, proc, file, line, func);
	kprintf(str);
}


void kprintf1(char *format, int arg1) {
	char str[512];

	sprintf(str, format, arg1);
	kprintf(str);
}

void kprintf2(char *format, int arg1, int arg2) {
	char str[512];

	sprintf(str, format, arg1, arg2);
	kprintf(str);
}

void kprintf3(char *format, int arg1, int arg2, int arg3) {
	char str[512];

	sprintf(str, format, arg1, arg2, arg3);
	kprintf(str);
}

#endif /* TRACE */

